/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Definitions from the Linux kernel's private header
 * drivers/hid/hid-ids.h
 */
#ifndef HID_IDS_H_FILE
#define HID_IDS_H_FILE

#define USB_VENDOR_ID_HUION		0x256c
#define USB_DEVICE_ID_HUION_TABLET	0x006e
#define USB_DEVICE_ID_HUION_TABLET2	0x006d

#define USB_VENDOR_ID_KYE		0x0458
#define USB_DEVICE_ID_KYE_ERGO_525V	0x0087
#define USB_DEVICE_ID_GENIUS_GILA_GAMING_MOUSE	0x0138
#define USB_DEVICE_ID_GENIUS_MANTICORE	0x0153
#define USB_DEVICE_ID_GENIUS_GX_IMPERATOR	0x4018
#define USB_DEVICE_ID_KYE_GPEN_560	0x5003
#define USB_DEVICE_ID_KYE_EASYPEN_I405X	0x5010
#define USB_DEVICE_ID_KYE_MOUSEPEN_I608X	0x5011
#define USB_DEVICE_ID_KYE_MOUSEPEN_I608X_V2	0x501a
#define USB_DEVICE_ID_KYE_EASYPEN_M610X	0x5013
#define USB_DEVICE_ID_KYE_PENSKETCH_M912	0x5015
#define USB_DEVICE_ID_KYE_EASYPEN_M406XE	0x5019

#define USB_VENDOR_ID_UCLOGIC		0x5543
#define USB_DEVICE_ID_UCLOGIC_TABLET_PF1209	0x0042
#define USB_DEVICE_ID_UCLOGIC_TABLET_KNA5	0x6001
#define USB_DEVICE_ID_UCLOGIC_TABLET_WP4030U	0x0003
#define USB_DEVICE_ID_UCLOGIC_TABLET_WP5540U	0x0004
#define USB_DEVICE_ID_UCLOGIC_TABLET_WP8060U	0x0005
#define USB_DEVICE_ID_UCLOGIC_TABLET_WP1062	0x0064
#define USB_DEVICE_ID_UCLOGIC_WIRELESS_TABLET_TWHL850	0x0522
#define USB_DEVICE_ID_UCLOGIC_TABLET_TWHA60	0x0781
#define USB_DEVICE_ID_UCLOGIC_DRAWIMAGE_G3	0x3031

#define USB_DEVICE_ID_UCLOGIC_UGEE_TABLET_81	0x0081
#define USB_DEVICE_ID_UCLOGIC_UGEE_TABLET_45	0x0045
#define USB_DEVICE_ID_UCLOGIC_UGEE_TABLET_47	0x0047

#define USB_DEVICE_ID_YIYNOVA_TABLET	0x004d

/* Known as Zippy Technology Corp. in usbutils */
#define USB_VENDOR_ID_POLOSTAR 0x099a
#define USB_DEVICE_ID_POLOSTAR_TABLET_PT1001 0x2620

/* UGTizer Technology Corp. */
#define USB_VENDOR_ID_UGTIZER			0x2179
#define USB_DEVICE_ID_UGTIZER_TABLET_GP0610	0x0053

/* Ugee Technology / XP-Pen Technology CO. */
#define USB_VENDOR_ID_UGEE			0x28bd
#define USB_DEVICE_ID_UGEE_XPPEN_TABLET_G540	0x0075
#define USB_DEVICE_ID_UGEE_XPPEN_TABLET_G640	0x0094
#define USB_DEVICE_ID_UGEE_XPPEN_TABLET_DECO01	0x0042
#define USB_DEVICE_ID_UGEE_XPPEN_TABLET_STAR06	0x0078
#define USB_DEVICE_ID_UGEE_TABLET_G5		0x0074
#define USB_DEVICE_ID_UGEE_TABLET_EX07S		0x0071
#define USB_DEVICE_ID_UGEE_TABLET_RAINBOW_CV720	0x0055

/* ViewSonic Corporation */
#define USB_VENDOR_ID_VIEWSONIC			0x0543
#define USB_DEVICE_ID_VIEWSONIC_PD1011		0xe621

/* Signotec GmbH */
#define USB_VENDOR_ID_SIGNOTEC			0x2133
#define USB_DEVICE_ID_SIGNOTEC_VIEWSONIC_PD1011	0x0018

#endif
